@@ -24,6 +24,7 @@ from debusine.artifacts.models import (
     CollectionCategory,
     TaskTypes,
 )
+from debusine.tasks.models import BackendType
 from utils.client import Client
 from utils.common import Configuration
 from utils.integration_test_helpers_mixin import IntegrationTestHelpersMixin
@@ -361,17 +362,18 @@ class IntegrationWorkflowDebianPipelineTests(
             "vendor": "debian",
             "codename": "trixie",
             "architectures": [self.architecture],
-            "sbuild_backend": "unshare",
+            "sbuild_backend": BackendType.UNSHARE,
             "qa_suite": f"{suite_name}@{CollectionCategory.SUITE}",
-            "autopkgtest_backend": "unshare",
-            "lintian_backend": "unshare",
-            "piuparts_backend": "unshare",
+            "autopkgtest_backend": BackendType.UNSHARE,
+            "lintian_backend": BackendType.UNSHARE,
+            "piuparts_backend": BackendType.UNSHARE,
             "enable_debdiff": True,
         }
         workflow_template_result = Client.execute_command(
-            "create-workflow-template",
-            template_name,
+            "workflow-template",
+            "create",
             "debian_pipeline",
+            template_name,
             stdin=yaml.safe_dump(template_data),
         )
         assert workflow_template_result.returncode == 0
@@ -421,9 +423,10 @@ class IntegrationWorkflowDebianPipelineTests(
             "publish_target": f"{suite_name}@{CollectionCategory.SUITE}",
         }
         workflow_template_result = Client.execute_command(
-            "create-workflow-template",
-            template_name,
+            "workflow-template",
+            "create",
             "debian_pipeline",
+            template_name,
             stdin=yaml.safe_dump(template_data),
         )
         assert workflow_template_result.returncode == 0
