@@ -44,7 +44,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
         """Set up stricter pydantic Config."""
 
         validate_assignment = True
-        extra = pydantic.Extra.forbid
+        extra = "forbid"
 
     #: Artifact type
     category: data_models.ArtifactCategory
@@ -927,9 +927,13 @@ class RepositoryIndex(LocalArtifact[data_models.DebianRepositoryIndex]):
     _category = data_models.ArtifactCategory.REPOSITORY_INDEX
 
     @classmethod
-    def create(cls, *, file: Path, path: str) -> Self:
+    def create(
+        cls, *, file: Path, path: str, content_type: str | None = None
+    ) -> Self:
         """Return a RepositoryIndex."""
         data = data_models.DebianRepositoryIndex(path=path)
         artifact = cls(category=cls._category, data=data)
-        artifact.add_local_file(file, override_name=PurePath(path).name)
+        artifact.add_local_file(
+            file, override_name=PurePath(path).name, content_type=content_type
+        )
         return artifact
