@@ -16,9 +16,8 @@ import re
 import secrets
 import urllib.parse
 from pathlib import Path
-from typing import Any, NamedTuple, Self, assert_never
+from typing import Any, assert_never
 
-import configobj
 import requests
 import rich.box
 from configobj import ConfigObj
@@ -27,7 +26,7 @@ from rich.prompt import Confirm
 from rich.table import Table
 from xkcdpass import xkcd_password
 
-from debusine.client.config import ConfigHandler
+from debusine.client.config import ConfigHandler, KNOWN_SERVERS, ServerInfo
 from debusine.client.dataentry import DataEntry
 from debusine.client.models import (
     EnrollConfirmPayload,
@@ -40,231 +39,92 @@ class TokenFetchFailed(Exception):
     """Exception raised when fetching a token failed."""
 
 
-class ServerInfo(NamedTuple):
-    """Information about a known server."""
+def fetch_token(console: Console, server: ServerInfo) -> str | None:
+    """Get an API token from the server."""
+    # Load data for generating challenges
+    xkcd_wordfile = xkcd_password.locate_wordfile()
+    xkcd_words = xkcd_password.generate_wordlist(
+        wordfile=xkcd_wordfile, min_length=5, max_length=8
+    )
 
-    #: Server name to use in configuration
-    name: str
-    #: User-visible description
-    desc: str
-    #: Default API URL
-    api_url: str
-    #: Default Scope
-    scope: str
-    #: API token
-    api_token: str | None = None
+    payload = EnrollPayload(
+        # Nonce used to identify the enrolling session
+        nonce=secrets.token_urlsafe(16),
+        # Challenge text
+        challenge=xkcd_password.generate_xkcdpassword(xkcd_words),
+        scope=server.scope,
+        hostname=platform.node(),
+    )
 
-    def lint(self) -> list[str]:
-        """
-        Run consistency checks on the current draft configuration.
+    api_url_parsed = urllib.parse.urlparse(server.api_url)
 
-        :returns: a list of messages, or empty if everything looks ordinary
-        """
-        res: list[str] = []
+    # URL used to get the token
+    enroll_url_parsed = api_url_parsed._replace(
+        path=os.path.join(api_url_parsed.path, "enroll/")
+    )
+    enroll_url = urllib.parse.urlunparse(enroll_url_parsed)
 
-        api_url_parsed = urllib.parse.urlparse(self.api_url)
-
-        if api_url_parsed.hostname == "localhost":
-            expected_scheme = "http"
-        else:
-            expected_scheme = "https"
-
-        if api_url_parsed.scheme != expected_scheme:
-            res.append(
-                f"Server url is an [bold]{api_url_parsed.scheme}[/] url,"
-                f" should it be [bold]{expected_scheme}[/]?"
-            )
-
-        if not self.api_url.endswith("/api") and not self.api_url.endswith(
-            "/api/"
-        ):
-            res.append("API URL does not end in /api or /api/")
-
-        return res
-
-    @classmethod
-    def from_config(cls, name: str, section: configobj.Section) -> Self:
-        """Infer a ServerInfo from an existing configuration."""
-        inferred = cls.from_string(name, desc="temporary serverinfo")
-        return cls(
-            name=name,
-            desc=f"Configured in {os.path.basename(section.main.filename)}",
-            api_url=section.get("api-url", inferred.api_url),
-            scope=section.get("scope", inferred.scope),
-            api_token=section.get("token", None),
-        )
-
-    @classmethod
-    def from_string(cls, name: str, desc: str) -> Self | "ServerInfo":
-        """Infer a ServerInfo from a user-provided name."""
-        if ":" not in name:
-            # plain hostname
-            server_url_parsed = urllib.parse.ParseResult(
-                scheme="",
-                netloc=name,
-                path="",
-                params="",
-                query="",
-                fragment="",
-            )
-            server_name = name
-        elif re.match(r"^[^/]*:\d+(?:/|$)", name):
-            # hostname:port, or just :port
-            server_url_parsed = urllib.parse.ParseResult(
-                scheme="",
-                netloc=(f"localhost{name}" if name.startswith(":") else name),
-                path="",
-                params="",
-                query="",
-                fragment="",
-            )
-            assert server_url_parsed.hostname
-            server_name = server_url_parsed.hostname
-        else:
-            # URL
-            server_url_parsed = urllib.parse.urlparse(name)
-            assert server_url_parsed.hostname
-            server_name = server_url_parsed.hostname
-
-        # Lookup known names
-        assert server_url_parsed.hostname
-        if known := KNOWN_SERVERS.get(server_url_parsed.hostname):
-            return known
-        # Lookup known hostnames
-        for info in KNOWN_SERVERS.values():
-            info_parsed = urllib.parse.urlparse(info.api_url)
-            if info_parsed.hostname == server_url_parsed.hostname:
-                return info
-
-        if not server_url_parsed.scheme:
-            server_url_parsed = server_url_parsed._replace(scheme="https")
-
-        if os.path.basename(server_url_parsed.path.rstrip("/")) != "api":
-            api_url_parsed = server_url_parsed._replace(
-                path=os.path.join(server_url_parsed.path, "api")
-            )
-        else:
-            api_url_parsed = server_url_parsed
-
-        api_url = urllib.parse.urlunparse(api_url_parsed)
-        if not api_url.endswith("/"):
-            api_url += "/"
-
-        return cls(
-            name=server_name, desc=desc, api_url=api_url, scope="debusine"
-        )
-
-    def fetch_token(self, console: Console) -> str | None:
-        """Get an API token from the server."""
-        # Load data for generating challenges
-        xkcd_wordfile = xkcd_password.locate_wordfile()
-        xkcd_words = xkcd_password.generate_wordlist(
-            wordfile=xkcd_wordfile, min_length=5, max_length=8
-        )
-
-        payload = EnrollPayload(
-            # Nonce used to identify the enrolling session
-            nonce=secrets.token_urlsafe(16),
-            # Challenge text
-            challenge=xkcd_password.generate_xkcdpassword(xkcd_words),
-            scope=self.scope,
-            hostname=platform.node(),
-        )
-
-        api_url_parsed = urllib.parse.urlparse(self.api_url)
-
-        # URL used to get the token
-        enroll_url_parsed = api_url_parsed._replace(
-            path=os.path.join(api_url_parsed.path, "enroll/")
-        )
-        enroll_url = urllib.parse.urlunparse(enroll_url_parsed)
-
-        # URL used for the user to confirm enrollment
-        confirm_url_parsed = api_url_parsed._replace(
-            path=os.path.normpath(
-                os.path.join(
-                    api_url_parsed.path, f"../-/enroll/confirm/{payload.nonce}/"
-                )
+    # URL used for the user to confirm enrollment
+    confirm_url_parsed = api_url_parsed._replace(
+        path=os.path.normpath(
+            os.path.join(
+                api_url_parsed.path, f"../-/enroll/confirm/{payload.nonce}/"
             )
         )
-        confirm_url = urllib.parse.urlunparse(confirm_url_parsed)
+    )
+    confirm_url = urllib.parse.urlunparse(confirm_url_parsed)
 
-        console.print(
-            ":point_right:",
-            "Please visit",
-            f"[link={confirm_url}]{rich.markup.escape(confirm_url)}[/]",
-            "to confirm registration",
-        )
-        console.print(
-            ":point_right:",
-            "Make sure the page mentions the passphrase:"
-            f' "[bold]{payload.challenge}[/]"',
-        )
+    console.print(
+        ":point_right:",
+        "Please visit",
+        f"[link={confirm_url}]{rich.markup.escape(confirm_url)}[/]",
+        "to confirm registration",
+    )
+    console.print(
+        ":point_right:",
+        "Make sure the page mentions the passphrase:"
+        f' "[bold]{payload.challenge}[/]"',
+    )
 
-        payload_dict = payload.dict()
-        try:
-            response = requests.post(enroll_url, json=payload_dict)
-        except Exception as e:
-            raise TokenFetchFailed(f"request to server failed: {e}") from e
-
-        if response.status_code != 200:
-            lead = f"The server returned status code {response.status_code}: "
-            try:
-                result = response.json()
-            except requests.exceptions.JSONDecodeError:
-                raise TokenFetchFailed(lead + response.text)
-            else:
-                if message := result.get("title"):
-                    raise TokenFetchFailed(lead + message)
-                else:
-                    raise TokenFetchFailed(lead + json.dumps(result))
+    payload_dict = payload.dict()
+    try:
+        response = requests.post(enroll_url, json=payload_dict)
+    except Exception as e:
+        raise TokenFetchFailed(f"request to server failed: {e}") from e
 
+    if response.status_code != 200:
+        lead = f"The server returned status code {response.status_code}: "
         try:
             result = response.json()
         except requests.exceptions.JSONDecodeError:
-            raise TokenFetchFailed("Invalid JSON in response")
+            raise TokenFetchFailed(lead + response.text)
+        else:
+            if message := result.get("title"):
+                raise TokenFetchFailed(lead + message)
+            else:
+                raise TokenFetchFailed(lead + json.dumps(result))
 
-        try:
-            confirmation = EnrollConfirmPayload.parse_obj(result)
-        except ValueError:
-            raise TokenFetchFailed("Invalid response payload")
+    try:
+        result = response.json()
+    except requests.exceptions.JSONDecodeError:
+        raise TokenFetchFailed("Invalid JSON in response")
 
-        match confirmation.outcome:
-            case EnrollOutcome.CONFIRM:
-                console.print(
-                    ":white_check_mark: Confirmation confirmed, token acquired"
-                )
-                return confirmation.token
-            case EnrollOutcome.CANCEL:
-                console.print(":exclamation_mark: Confirmation cancelled")
-                return None
-            case _ as unreachable:
-                assert_never(unreachable)
+    try:
+        confirmation = EnrollConfirmPayload.parse_obj(result)
+    except ValueError:
+        raise TokenFetchFailed("Invalid response payload")
 
-
-KNOWN_SERVERS: dict[str, ServerInfo] = {
-    info.name: info
-    for info in (
-        ServerInfo(
-            name="debian",
-            desc="debusine.debian.net",
-            api_url="https://debusine.debian.net/api/",
-            scope="debian",
-        ),
-        ServerInfo(
-            name="freexian",
-            desc="Freexian (E)LTS",
-            api_url="https://debusine.freexian.com/api/",
-            scope="freexian",
-        ),
-        ServerInfo(
-            name="localhost",
-            desc="debusine-admin runserver for development",
-            api_url="http://localhost:8000/api/",
-            scope="debusine",
-        ),
-    )
-}
+    match confirmation.outcome:
+        case EnrollOutcome.CONFIRM:
+            console.print(
+                ":white_check_mark: Confirmation confirmed, token acquired"
+            )
+            return confirmation.token
+        case EnrollOutcome.CANCEL:
+            console.print(":exclamation_mark: Confirmation cancelled")
+            return None
+        case _ as unreachable:
+            assert_never(unreachable)
 
 
 class DebusineClientConfig:
@@ -283,7 +143,7 @@ class DebusineClientConfig:
             list_values=False,
         )
         self.servers: dict[str, ServerInfo] = {
-            name[7:]: ServerInfo.from_config(name[7:], section)
+            name[7:]: ServerInfo.from_config(name[7:], section, path)
             for name, section in self.config.items()
             if name.startswith("server:")
         }
@@ -347,10 +207,16 @@ class DebusineClientConfig:
         """Set the current server as default server."""
         self.default_server = self.server.name
 
+    def set_default_workspace(self, workspace: str) -> None:
+        """Set the default workspace of the current server."""
+        edited = self.server._replace(default_workspace=workspace)
+        self.current_server = edited
+        self.edited[edited.name] = edited
+
     def fetch_token(self, console: Console) -> bool:
         """Fetch a token and set it in the current configuration."""
         try:
-            api_token = self.server.fetch_token(console)
+            api_token = fetch_token(console, self.server)
         except TokenFetchFailed as e:
             console.print(":exclamation_mark:", rich.markup.escape(str(e)))
             return False
@@ -379,6 +245,11 @@ class DebusineClientConfig:
                         del section["token"]
                 else:
                     section["token"] = current.api_token
+            if (
+                orig is None
+                or orig.default_workspace != current.default_workspace
+            ):
+                section["default-workspace"] = current.default_workspace
 
         if self.default_server is not None:
             if section := self.config.get("General"):
@@ -439,6 +310,10 @@ class ServerConfigEditor(DataEntry):
             + "[/]",
             "([bold]t[/]oken)",
         )
+        table.add_row(
+            f"default-workspace = [bold]{server.default_workspace}[bold]",
+            "([bold]w[/]orkspace)",
+        )
 
         self.console.print(table)
 
@@ -480,6 +355,17 @@ class ServerConfigEditor(DataEntry):
             or None
         )
 
+    def do_workspace(self, arg: str) -> None:  # noqa: U100
+        """Change the default workspace."""
+        server = self.config.server
+        self.config.set_default_workspace(
+            self.input_line(
+                "Enter default workspace name",
+                initial=server.default_workspace,
+                required=True,
+            )
+        )
+
     def do_quit(self, arg: str) -> bool:  # noqa: U100
         """Quit editor."""
         self.console.print(
