@@ -19,6 +19,7 @@ from unittest import mock
 import responses
 from configobj import ConfigObj
 
+from debusine.client.config import KNOWN_SERVERS, ServerInfo
 from debusine.client.models import (
     EnrollConfirmPayload,
     EnrollOutcome,
@@ -26,11 +27,10 @@ from debusine.client.models import (
 )
 from debusine.client.setup import (
     DebusineClientConfig,
-    KNOWN_SERVERS,
     ServerConfigEditor,
-    ServerInfo,
     ServerSelector,
     TokenFetchFailed,
+    fetch_token,
     setup_server,
 )
 from debusine.client.tests.utils import BufferConsole, MockTable
@@ -47,7 +47,7 @@ class ServerInfoTests(TestCase):
         """Test from_config."""
         workdir = self.create_temporary_directory()
         path = workdir / "debusine.ini"
-        desc = f"Configured in {path.name}"
+        desc = f"Configured in {path}"
         for name, cfg, info in (
             (
                 "example",
@@ -57,6 +57,22 @@ class ServerInfoTests(TestCase):
                     api_url="http://example.org/api/",
                     desc=desc,
                     scope="debusine",
+                    default_workspace="System",
+                ),
+            ),
+            (
+                "example",
+                {
+                    "api-url": "http://example.org/api/",
+                    "scope": "debusine",
+                    "default-workspace": "dw",
+                },
+                ServerInfo(
+                    name="example",
+                    api_url="http://example.org/api/",
+                    desc=desc,
+                    scope="debusine",
+                    default_workspace="dw",
                 ),
             ),
             (
@@ -67,6 +83,7 @@ class ServerInfoTests(TestCase):
                     api_url="https://debusine.debian.net/api/",
                     desc=desc,
                     scope="debusine",
+                    default_workspace="developers",
                 ),
             ),
             (
@@ -77,6 +94,7 @@ class ServerInfoTests(TestCase):
                     api_url="https://empty/api/",
                     desc=desc,
                     scope="debusine",
+                    default_workspace="System",
                 ),
             ),
         ):
@@ -85,9 +103,9 @@ class ServerInfoTests(TestCase):
             initial_cfg.write()
             loaded_cfg = ConfigObj(path.as_posix())
             loaded_info = ServerInfo.from_config(
-                name, loaded_cfg[f"server:{name}"]
+                name, loaded_cfg[f"server:{name}"], path
             )
-            self.assertEqual(loaded_info, info)
+            assert loaded_info == info
 
     def test_from_string_known(self) -> None:
         """Test from_string with a known name or hostname."""
@@ -165,6 +183,7 @@ class ServerInfoTests(TestCase):
                         api_url=api_url,
                         scope="debusine",
                         desc="desc",
+                        default_workspace="System",
                     ),
                 )
 
@@ -228,7 +247,7 @@ class ServerInfoTests(TestCase):
             ),
         ):
             try:
-                result = server.fetch_token(console)
+                result = fetch_token(console, server)
             except TokenFetchFailed as e:
                 exception = e
                 result = None
@@ -479,6 +498,7 @@ class DebusineClientConfigTests(TestCase):
                 [server:localhost]
                 api-url = http://localhost:8000/api/
                 scope = debusine
+                default-workspace = Playground
                 """
             ),
         )
@@ -527,6 +547,7 @@ class DebusineClientConfigTests(TestCase):
         [server:localhost]
         api-url = http://localhost:8000/api/
         scope = debusine
+        default-workspace = Playground
         """
             ),
         )
@@ -552,6 +573,7 @@ class DebusineClientConfigTests(TestCase):
                 [server:localhost]
                 api-url = http://localhost:8000/api/
                 scope = debusine
+                default-workspace = Playground
                 """,
             ),
         )
@@ -573,6 +595,7 @@ class DebusineClientConfigTests(TestCase):
                     [server:localhost]
                     api-url = http://localhost:8000/api/
                     scope = debusine
+                    default-workspace = Playground
                     """
                 ),
             ),
@@ -601,6 +624,7 @@ class DebusineClientConfigTests(TestCase):
                     [server:localhost]
                     api-url = http://localhost:8000/api/
                     scope = debusine
+                    default-workspace = Playground
                     """
                 ),
             ),
@@ -702,7 +726,7 @@ class DebusineClientConfigTests(TestCase):
         console = BufferConsole()
         cfg = self.make_config()
         with mock.patch(
-            "debusine.client.setup.ServerInfo.fetch_token", return_value=token
+            "debusine.client.setup.fetch_token", return_value=token
         ):
             self.assertTrue(cfg.fetch_token(console))
         self.assertEqual(cfg.server.api_token, token)
@@ -712,9 +736,7 @@ class DebusineClientConfigTests(TestCase):
         console = BufferConsole()
         cfg = self.make_config()
         cfg.set_api_token("aaaaaa")
-        with mock.patch(
-            "debusine.client.setup.ServerInfo.fetch_token", return_value=None
-        ):
+        with mock.patch("debusine.client.setup.fetch_token", return_value=None):
             self.assertTrue(cfg.fetch_token(console))
         self.assertIsNone(cfg.server.api_token)
         self.assertEqual(console.output_lines(), [])
@@ -725,7 +747,7 @@ class DebusineClientConfigTests(TestCase):
         cfg = self.make_config()
         cfg.set_api_token(token)
         with mock.patch(
-            "debusine.client.setup.ServerInfo.fetch_token",
+            "debusine.client.setup.fetch_token",
             side_effect=TokenFetchFailed("message"),
         ):
             self.assertFalse(cfg.fetch_token(console))
@@ -775,6 +797,7 @@ class DebusineClientConfigTests(TestCase):
                 "[server:debian]",
                 "api-url = https://debusine.debian.net/api/",
                 "scope = testdebian",
+                "default-workspace = developers",
             ],
         )
 
@@ -881,7 +904,7 @@ class ServerSelectorTests(TestCase):
                 "1…3, new, quit:",
             ),
         ):
-            with self.subTest(entries=entries):
+            with self.subTest(entries=repr(entries)):
                 console = BufferConsole()
                 selector = ServerSelector(entries, console=console)
                 selector.menu()
@@ -958,6 +981,10 @@ class ServerConfigEditorTests(TestCase):
                     "([bold]u[/]rl)",
                 ),
                 ("token = [italic]acquired on save[/]", "([bold]t[/]oken)"),
+                (
+                    "default-workspace = [bold]developers[bold]",
+                    "([bold]w[/]orkspace)",
+                ),
             ],
         )
 
@@ -1053,6 +1080,26 @@ class ServerConfigEditorTests(TestCase):
             "Enter (or clear) API token", initial=None
         )
 
+    def test_edit_default_workspace(self) -> None:
+        """Edit the api url."""
+        old_default_workspace = "developers"
+        new_default_workspace = "Changed"
+        cfg = self._config()
+        self.assertEqual(cfg.server.default_workspace, old_default_workspace)
+        console = BufferConsole()
+        editor = ServerConfigEditor(cfg, console=console)
+        with mock.patch(
+            "debusine.client.dataentry.DataEntry.input_line",
+            return_value=new_default_workspace,
+        ) as input_line:
+            editor.onecmd("workspace")
+        self.assertEqual(cfg.server.default_workspace, new_default_workspace)
+        input_line.assert_called_with(
+            "Enter default workspace name",
+            initial=old_default_workspace,
+            required=True,
+        )
+
     def test_set_default(self) -> None:
         """Set the current server as default."""
         cfg = self._config()
