@@ -19,11 +19,13 @@ from debusine.server.tasks.wait.base import BaseWaitTask  # isort: split
 # Sub-tasks need to be imported in order to be available to BaseTask
 # (e.g. for BaseTask.is_valid_task_name). They are registered via
 # BaseTask.__init_subclass__.
+from debusine.server.tasks.wait.confirm import Confirm
 from debusine.server.tasks.wait.delay import Delay
 from debusine.server.tasks.wait.external_debsign import ExternalDebsign
 
 __all__ = [
     "BaseWaitTask",
+    "Confirm",
     "Delay",
     "ExternalDebsign",
 ]
