@@ -9,6 +9,10 @@
 
 """qa workflow."""
 
+from django.db.models import Q, QuerySet
+
+from debusine.artifacts.models import TaskTypes
+from debusine.db.models import WorkRequest
 from debusine.server.workflows import workflow_utils
 from debusine.server.workflows.models import (
     AutopkgtestWorkflowData,
@@ -16,6 +20,7 @@ from debusine.server.workflows.models import (
     DebDiffWorkflowData,
     LintianWorkflowData,
     PiupartsWorkflowData,
+    QAFailOn,
     QAWorkflowData,
     ReverseDependenciesAutopkgtestWorkflowData,
     WorkRequestWorkflowData,
@@ -29,6 +34,8 @@ from debusine.tasks.models import (
     LintianFailOnSeverity,
     LookupMultiple,
     LookupSingle,
+    OutputData,
+    RegressionAnalysisStatus,
 )
 from debusine.tasks.server import TaskDatabaseInterface
 
@@ -81,19 +88,27 @@ class QAWorkflow(
             )
         )
 
+        allow_failure = self.data.fail_on in {
+            QAFailOn.NEVER,
+            QAFailOn.REGRESSION,
+        }
+
         if self.data.enable_autopkgtest:
             self._populate_autopkgtest(
                 prefix=self.data.prefix,
+                reference_prefix=self.data.reference_prefix,
                 source_artifact=self.data.source_artifact,
                 binary_artifacts=filtered_binary_artifacts,
                 qa_suite=self.data.qa_suite,
                 reference_qa_results=self.data.reference_qa_results,
+                enable_regression_tracking=self.data.enable_regression_tracking,
                 update_qa_results=self.data.update_qa_results,
                 vendor=self.data.vendor,
                 codename=self.data.codename,
                 backend=self.data.autopkgtest_backend,
                 architectures=effective_architectures,
                 arch_all_host_architecture=self.data.arch_all_host_architecture,
+                allow_failure=allow_failure,
             )
 
         if self.data.enable_reverse_dependencies_autopkgtest:
@@ -102,25 +117,30 @@ class QAWorkflow(
             assert self.data.qa_suite is not None
             self._populate_reverse_dependencies_autopkgtest(
                 prefix=self.data.prefix,
+                reference_prefix=self.data.reference_prefix,
                 source_artifact=self.data.source_artifact,
                 binary_artifacts=filtered_binary_artifacts,
                 qa_suite=self.data.qa_suite,
                 reference_qa_results=self.data.reference_qa_results,
+                enable_regression_tracking=self.data.enable_regression_tracking,
                 update_qa_results=self.data.update_qa_results,
                 vendor=self.data.vendor,
                 codename=self.data.codename,
                 backend=self.data.autopkgtest_backend,
                 architectures=effective_architectures,
                 arch_all_host_architecture=self.data.arch_all_host_architecture,
+                allow_failure=allow_failure,
             )
 
         if self.data.enable_lintian:
             self._populate_lintian(
                 prefix=self.data.prefix,
+                reference_prefix=self.data.reference_prefix,
                 source_artifact=self.data.source_artifact,
                 binary_artifacts=filtered_binary_artifacts,
                 qa_suite=self.data.qa_suite,
                 reference_qa_results=self.data.reference_qa_results,
+                enable_regression_tracking=self.data.enable_regression_tracking,
                 update_qa_results=self.data.update_qa_results,
                 vendor=self.data.vendor,
                 codename=self.data.codename,
@@ -128,15 +148,18 @@ class QAWorkflow(
                 architectures=effective_architectures,
                 arch_all_host_architecture=self.data.arch_all_host_architecture,
                 fail_on_severity=self.data.lintian_fail_on_severity,
+                allow_failure=allow_failure,
             )
 
         if self.data.enable_piuparts:
             self._populate_piuparts(
                 prefix=self.data.prefix,
+                reference_prefix=self.data.reference_prefix,
                 source_artifact=self.data.source_artifact,
                 binary_artifacts=filtered_binary_artifacts,
                 qa_suite=self.data.qa_suite,
                 reference_qa_results=self.data.reference_qa_results,
+                enable_regression_tracking=self.data.enable_regression_tracking,
                 update_qa_results=self.data.update_qa_results,
                 vendor=self.data.vendor,
                 codename=self.data.codename,
@@ -144,6 +167,7 @@ class QAWorkflow(
                 backend=self.data.piuparts_backend,
                 environment=self.data.piuparts_environment,
                 arch_all_host_architecture=self.data.arch_all_host_architecture,
+                allow_failure=allow_failure,
             )
 
         if self.data.enable_debdiff:
@@ -156,6 +180,7 @@ class QAWorkflow(
                 vendor=self.data.vendor,
                 codename=self.data.codename,
                 original=self.data.qa_suite,
+                allow_failure=allow_failure,
             )
 
         if self.data.enable_blhc:
@@ -164,39 +189,51 @@ class QAWorkflow(
             assert self.data.package_build_logs
             self._populate_blhc(
                 prefix=self.data.prefix,
+                reference_prefix=self.data.reference_prefix,
+                source_artifact=self.data.source_artifact,
                 package_build_logs=self.data.package_build_logs,
                 qa_suite=self.data.qa_suite,
                 reference_qa_results=self.data.reference_qa_results,
+                enable_regression_tracking=self.data.enable_regression_tracking,
                 update_qa_results=self.data.update_qa_results,
                 vendor=self.data.vendor,
                 codename=self.data.codename,
                 arch_all_host_architecture=self.data.arch_all_host_architecture,
+                allow_failure=allow_failure,
             )
 
+        if self.data.fail_on == QAFailOn.REGRESSION:
+            self._populate_regression_analysis_callback()
+
     def _populate_autopkgtest(
         self,
         *,
         prefix: str,
+        reference_prefix: str,
         source_artifact: LookupSingle,
         binary_artifacts: LookupMultiple,
         qa_suite: LookupSingle | None,
         reference_qa_results: LookupSingle | None,
+        enable_regression_tracking: bool,
         update_qa_results: bool,
         vendor: str,
         codename: str,
         backend: BackendType,
         architectures: list[str],
         arch_all_host_architecture: str,
+        allow_failure: bool,
     ) -> None:
         """Create work request for autopkgtest workflow."""
         wr = self.work_request_ensure_child_workflow(
             task_name="autopkgtest",
             task_data=AutopkgtestWorkflowData(
                 prefix=prefix,
+                reference_prefix=reference_prefix,
                 source_artifact=source_artifact,
                 binary_artifacts=binary_artifacts,
                 qa_suite=qa_suite,
                 reference_qa_results=reference_qa_results,
+                enable_regression_tracking=enable_regression_tracking,
                 update_qa_results=update_qa_results,
                 vendor=vendor,
                 codename=codename,
@@ -208,6 +245,7 @@ class QAWorkflow(
             workflow_data=WorkRequestWorkflowData(
                 display_name="autopkgtest",
                 step="autopkgtest",
+                allow_failure=allow_failure,
             ),
         )
         # The autopkgtest workflow's children will have dependencies on the
@@ -220,26 +258,31 @@ class QAWorkflow(
         self,
         *,
         prefix: str,
+        reference_prefix: str,
         source_artifact: LookupSingle,
         binary_artifacts: LookupMultiple,
         qa_suite: LookupSingle,
         reference_qa_results: LookupSingle | None,
+        enable_regression_tracking: bool,
         update_qa_results: bool,
         vendor: str,
         codename: str,
         backend: BackendType,
         architectures: list[str],
         arch_all_host_architecture: str,
+        allow_failure: bool,
     ) -> None:
         """Create work request for reverse_dependencies_autopkgtest workflow."""
         wr = self.work_request_ensure_child_workflow(
             task_name="reverse_dependencies_autopkgtest",
             task_data=ReverseDependenciesAutopkgtestWorkflowData(
                 prefix=prefix,
+                reference_prefix=reference_prefix,
                 source_artifact=source_artifact,
                 binary_artifacts=binary_artifacts,
                 qa_suite=qa_suite,
                 reference_qa_results=reference_qa_results,
+                enable_regression_tracking=enable_regression_tracking,
                 update_qa_results=update_qa_results,
                 vendor=vendor,
                 codename=codename,
@@ -251,6 +294,7 @@ class QAWorkflow(
             workflow_data=WorkRequestWorkflowData(
                 display_name="autopkgtests of reverse-dependencies",
                 step="reverse-dependencies-autopkgtest",
+                allow_failure=allow_failure,
             ),
         )
         # The reverse_dependencies_autopkgtest workflow's descendants will have
@@ -263,10 +307,12 @@ class QAWorkflow(
         self,
         *,
         prefix: str,
+        reference_prefix: str,
         source_artifact: LookupSingle,
         binary_artifacts: LookupMultiple,
         qa_suite: LookupSingle | None,
         reference_qa_results: LookupSingle | None,
+        enable_regression_tracking: bool,
         update_qa_results: bool,
         vendor: str,
         codename: str,
@@ -274,16 +320,19 @@ class QAWorkflow(
         architectures: list[str],
         arch_all_host_architecture: str,
         fail_on_severity: LintianFailOnSeverity,
+        allow_failure: bool,
     ) -> None:
         """Create work request for lintian workflow."""
         wr = self.work_request_ensure_child_workflow(
             task_name="lintian",
             task_data=LintianWorkflowData(
                 prefix=prefix,
+                reference_prefix=reference_prefix,
                 source_artifact=source_artifact,
                 binary_artifacts=binary_artifacts,
                 qa_suite=qa_suite,
                 reference_qa_results=reference_qa_results,
+                enable_regression_tracking=enable_regression_tracking,
                 update_qa_results=update_qa_results,
                 vendor=vendor,
                 codename=codename,
@@ -295,6 +344,7 @@ class QAWorkflow(
             workflow_data=WorkRequestWorkflowData(
                 display_name="lintian",
                 step="lintian",
+                allow_failure=allow_failure,
             ),
         )
         # The lintian workflow's descendants will have dependencies on the work
@@ -307,10 +357,12 @@ class QAWorkflow(
         self,
         *,
         prefix: str,
+        reference_prefix: str,
         source_artifact: LookupSingle,
         binary_artifacts: LookupMultiple,
         qa_suite: LookupSingle | None,
         reference_qa_results: LookupSingle | None,
+        enable_regression_tracking: bool,
         update_qa_results: bool,
         vendor: str,
         codename: str,
@@ -318,13 +370,16 @@ class QAWorkflow(
         backend: BackendType,
         environment: LookupSingle | None,
         arch_all_host_architecture: str,
+        allow_failure: bool,
     ) -> None:
         data = PiupartsWorkflowData(
             prefix=prefix,
+            reference_prefix=reference_prefix,
             source_artifact=source_artifact,
             binary_artifacts=binary_artifacts,
             qa_suite=qa_suite,
             reference_qa_results=reference_qa_results,
+            enable_regression_tracking=enable_regression_tracking,
             update_qa_results=update_qa_results,
             vendor=vendor,
             codename=codename,
@@ -341,6 +396,7 @@ class QAWorkflow(
             workflow_data=WorkRequestWorkflowData(
                 display_name="piuparts",
                 step="piuparts",
+                allow_failure=allow_failure,
             ),
         )
         # The piuparts workflow's descendants will have dependencies on the
@@ -356,6 +412,7 @@ class QAWorkflow(
         source_artifact: LookupSingle,
         binary_artifacts: LookupMultiple,
         original: LookupSingle,
+        allow_failure: bool,
     ) -> None:
         data = DebDiffWorkflowData(
             original=original,
@@ -371,6 +428,7 @@ class QAWorkflow(
             workflow_data=WorkRequestWorkflowData(
                 display_name="DebDiff",
                 step="debdiff",
+                allow_failure=allow_failure,
             ),
         )
 
@@ -385,28 +443,103 @@ class QAWorkflow(
         self,
         *,
         prefix: str,
+        reference_prefix: str,
+        source_artifact: LookupSingle,
         package_build_logs: LookupMultiple,
         qa_suite: LookupSingle | None,
         reference_qa_results: LookupSingle | None,
+        enable_regression_tracking: bool,
         update_qa_results: bool,
         vendor: str,
         codename: str,
         arch_all_host_architecture: str,
+        allow_failure: bool,
     ) -> None:
         wr = self.work_request_ensure_child_workflow(
             task_name="blhc",
             task_data=BlhcWorkflowData(
                 prefix=prefix,
+                reference_prefix=reference_prefix,
+                source_artifact=source_artifact,
                 package_build_logs=package_build_logs,
                 qa_suite=qa_suite,
                 reference_qa_results=reference_qa_results,
+                enable_regression_tracking=enable_regression_tracking,
                 update_qa_results=update_qa_results,
                 vendor=vendor,
                 codename=codename,
                 arch_all_host_architecture=arch_all_host_architecture,
             ),
             workflow_data=WorkRequestWorkflowData(
-                display_name="build log hardening check", step="blhc"
+                display_name="build log hardening check",
+                step="blhc",
+                allow_failure=allow_failure,
             ),
         )
         self.orchestrate_child(wr)
+
+    def _find_regression_analysis_sources(self) -> QuerySet[WorkRequest]:
+        """Find descendants that contribute regression analysis results."""
+        return WorkRequest.objects.filter(
+            Q(task_type=TaskTypes.WORKFLOW)
+            & (
+                # Most of the relevant tasks are children of this work
+                # request.
+                Q(parent=self.work_request)
+                # Reverse-dependency autopkgtests are grandchildren of this
+                # work request.
+                | Q(parent__parent=self.work_request)
+            )
+        )
+
+    def _populate_regression_analysis_callback(self) -> None:
+        """Create a final callback to collate regression analysis results."""
+        wr = self.work_request_ensure_child_internal(
+            task_name="workflow",
+            workflow_data=WorkRequestWorkflowData(
+                allow_failure=False,
+                display_name="Regression analysis",
+                step="regression-analysis",
+                visible=True,
+            ),
+        )
+        for callback in WorkRequest.objects.filter(
+            task_type=TaskTypes.INTERNAL,
+            task_name="workflow",
+            workflow_data_json__step="regression-analysis",
+            parent__in=self._find_regression_analysis_sources(),
+        ):
+            # Each individual regression analysis callback is a descendant
+            # of this work request, and therefore must be in the same
+            # workflow as the final callback.
+            wr.add_dependency(callback, skip_same_workflow_check=True)
+
+    def callback_regression_analysis(self) -> bool:
+        """Collate regression analysis results."""
+        output_data = self.work_request.output_data or OutputData()
+        output_data.regression_analysis = {}
+        overall_result = True
+        for (
+            task_name,
+            subject,
+            analysis_by_architecture,
+        ) in self._find_regression_analysis_sources().values_list(
+            "task_name",
+            "dynamic_task_data__subject",
+            "output_data_json__regression_analysis",
+        ):
+            for key, analysis in (analysis_by_architecture or {}).items():
+                # If we have output data, then dynamic task data must have
+                # been populated, and all our sub-workflows have a subject.
+                assert subject is not None
+                output_data.regression_analysis[
+                    f"{task_name}:{subject}:{key}"
+                ] = analysis
+                if analysis["status"] in {
+                    RegressionAnalysisStatus.ERROR,
+                    RegressionAnalysisStatus.REGRESSION,
+                }:
+                    overall_result = False
+        self.work_request.output_data = output_data
+        self.work_request.save()
+        return overall_result
