@@ -28,10 +28,7 @@ from debusine.db.models import (
 )
 from debusine.server.collections.lookup import lookup_single
 from debusine.server.workflows import DebDiffWorkflow, workflow_utils
-from debusine.server.workflows.base import (
-    WorkflowRunError,
-    orchestrate_workflow,
-)
+from debusine.server.workflows.base import orchestrate_workflow
 from debusine.server.workflows.debdiff import Architecture
 from debusine.server.workflows.models import (
     BaseWorkflowData,
@@ -47,6 +44,8 @@ from debusine.tasks.models import (
     DebDiffFlags,
     LookupMultiple,
     LookupSingle,
+    OutputData,
+    OutputDataError,
     SbuildData,
     SbuildInput,
 )
@@ -115,6 +114,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
         sbuild_architectures: list[str],
         source_artifact: Artifact,
         target_distribution: str,
+        expect_success: bool = True,
     ) -> WorkRequest:
         """Create and orchestrate a DebDiffWorkflow."""
 
@@ -160,7 +160,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
                 self.orchestrate_child(debdiff)
 
         root = self.playground.create_workflow(task_name="examplepipeline")
-        orchestrate_workflow(root)
+        self.assertEqual(orchestrate_workflow(root), expect_success)
 
         return root
 
@@ -175,6 +175,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
         arch_all_host_architecture: str | None = None,
         original: str | None = None,
         extra_flags: list[DebDiffFlags] | None = None,
+        expect_success: bool = True,
     ) -> WorkRequest:
         """
         Create and schedule a DebDiff workflow.
@@ -196,6 +197,8 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
         :param original: collection to use as "original". If it's None, use
           self.debian_collection
         :param extra_flags: if not None add "extra_flags" in the task data
+        :param expect_success: whether the orchestrator is expected to
+          succeed
         """
         if add_origin_source_artifact:
             self.debian_collection.manager.add_artifact(
@@ -261,6 +264,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
             sbuild_architectures=list(sbuild_archs),
             source_artifact=self.new_source_artifact,
             target_distribution="debian:bookworm",
+            expect_success=expect_success,
         )
         work_request = example_pipeline.children.get(
             task_name="debdiff", task_type=TaskTypes.WORKFLOW
@@ -958,5 +962,26 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
 
     @preserve_task_registry()
     def test_validate_input_invalid_original(self) -> None:
-        with self.assertRaisesRegex(WorkflowRunError, "non-existing"):
-            self.create_debdiff_workflow(original="non-existing")
+        work_request = self.create_debdiff_workflow(
+            original="non-existing", expect_success=False
+        )
+        # TODO: This should arguably be logged on the debdiff workflow
+        # itself, not on its parent.  At the moment,
+        # Workflow.orchestrate_child calls mark_pending which fails, and the
+        # exception is handled by the parent.
+        assert work_request.parent is not None
+        self.assertEqual(
+            work_request.parent.output_data,
+            OutputData(
+                errors=[
+                    OutputDataError(
+                        message=(
+                            "Orchestrator failed: 'non-existing' does not "
+                            "specify a category and the context does not "
+                            "supply a default"
+                        ),
+                        code="orchestrator-failed",
+                    )
+                ]
+            ),
+        )
