@@ -11,7 +11,7 @@
 import copy
 import shlex
 from pathlib import Path
-from typing import cast
+from typing import Any, cast
 from unittest import mock
 from unittest.mock import call
 
@@ -68,14 +68,23 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
             self.task._debug_log_files_directory.cleanup()
         super().tearDown()
 
+    def configure_task(
+        self,
+        task_data: dict[str, Any] | None = None,
+        override: dict[str, Any] | None = None,
+        remove: list[str] | None = None,
+    ) -> None:
+        """Perform further setup."""
+        super().configure_task(task_data, override, remove)
+        self.task_db = FakeTaskDatabase()
+
     def test_compute_dynamic_data(self) -> None:
         """Test compute_dynamic_data."""
-        task_database = FakeTaskDatabase()
-
         self.assertEqual(
-            self.task.compute_dynamic_data(task_database),
+            self.task.compute_dynamic_data(self.task_db),
             BaseDynamicTaskData(
                 subject="bookworm",
+                parameter_summary="bookworm (amd64)",
                 runtime_context="amd64:None:hello,python3",
                 configuration_context="amd64",
             ),
