@@ -14,8 +14,8 @@ Sidebar elements are informational elements or actions that can be used to
 populate the UI sidebar.
 """
 
+import datetime as dt
 from dataclasses import dataclass
-from datetime import datetime
 from functools import cached_property
 
 from django.db.models import Model
@@ -175,7 +175,7 @@ def create_user(
     )
 
 
-def create_created_at(at: datetime) -> SidebarItem:
+def create_created_at(at: dt.datetime) -> SidebarItem:
     """Create created_at information."""
     return SidebarElement(
         icon=Icons.CREATED_AT,
@@ -185,7 +185,7 @@ def create_created_at(at: datetime) -> SidebarItem:
     )
 
 
-def create_expire_at(at: datetime | None) -> SidebarItem:
+def create_expire_at(at: dt.datetime | None) -> SidebarItem:
     """Create expire_at information."""
     kwargs = {
         "icon": Icons.EXPIRE_AT,
