@@ -9,9 +9,9 @@
 
 """Test utility classes to help to test the Worker."""
 
+import datetime as dt
 from collections.abc import Callable
 from dataclasses import dataclass, field
-from datetime import datetime
 from enum import Enum, auto
 from typing import Any
 
@@ -187,7 +187,7 @@ class DebusineAioHTTPTestCase(AioHTTPTestCase):
                 id=52,
                 task_type="Worker",
                 task_name="sbuild",
-                created_at=datetime.fromisoformat(
+                created_at=dt.datetime.fromisoformat(
                     "2022-01-05T11:14:22.242178Z"
                 ),
                 started_at=None,
